import { useState } from "react";
import { CPUNAME, CPUS_NAMES } from "../lib/CPUS";
import { useSession } from "../lib/store";

export function UserSelection() {
    const session = useSession();

    const [username, setUsername] = useState('');

    if (session.currentUser) {
        return <div className="flex items-center justify-between p-2">
            <h2 className="text-xl font-bold">Hello, {session.currentUser}. Ready to play?</h2>
            <button type="button" onClick={() => session.setUser(undefined)}>
                Change User
            </button>
        </div>
    }

    return <>
        <form
            className="p-2 flex gap-5 items-center justify-center flex-wrap"
            onSubmit={(e) => {
                e.preventDefault();
                if (CPUS_NAMES.includes(e.currentTarget.value)) {
                    return;
                }
                session.setUser(username);
                setUsername('');
            }}
        >
            <h2 className="font-bold text-xl">Before we can start what is your name?</h2>
            <input
                value={username}
                className="border-b-white border-b-1 border-b-solid text-xl font-bold outline-none flex-grow md:max-w-[50ch]"
                onChange={(e) => {
                    if (CPUS_NAMES.includes(e.currentTarget.value as CPUNAME)) {
                        e.currentTarget.setCustomValidity('I am sorry, but that name is not valid');
                    } else {
                        e.currentTarget.setCustomValidity('');
                    }


                    setUsername(e.currentTarget.value);
                }}
                required
            />
            <button type="submit" className="p-1 bg-gray-400/10 rounded-lg px-2 w-full md:w-auto">
                Select
            </button>
        </form>
    </>;
}
