import { Play, PLAYER_OPTIONS, PlayerAction } from "../game";

export type CPUNAME = "CPU Random" | "CPU Always Rock" | "Slightly Smart";
export const CPUS_NAMES: CPUNAME[] = [
	"CPU Random",
	"CPU Always Rock",
	"Slightly Smart",
];

export const CPU_FUNCTION_MAPING: Record<
	CPUNAME,
	(plays: Play[]) => PlayerAction
> = {
	"CPU Random": CpuRandom,
	"CPU Always Rock": () => "Rock",
	"Slightly Smart": CpuSlightlySmart,
};

export const CPU_FUNCTION_MAPING_END: ((
	plays: Play[],
	user: string,
) => void)[] = [CpuSlightlySmartEnd];

function CpuRandom(): PlayerAction {
	return PLAYER_OPTIONS[Math.floor(Math.random() * PLAYER_OPTIONS.length)];
}

//
// CPU slightly Smart
//
// Keeps track of the all users plays and then use that to modify the chance
// of an action to be played. The action will be weigthed based on the actions
// that are likely to win agains the trend of what players have been playing
//
const CpuSlightlySmartValue = PLAYER_OPTIONS.reduce(
	(acc, a) => {
		acc[a] = 1;
		return acc;
	},
	{} as Record<PlayerAction, number>,
);
function CpuSlightlySmart(plays: Play[]): PlayerAction {
	const turn = plays.length;

	let base = CpuSlightlySmartValue;
	try {
		const newBase = JSON.parse(
			localStorage.getItem("slightly-smart-cpu-data") ?? "",
		);
		base = { ...base, ...newBase[turn] };
	} catch {
		// Do nothing use the default
	}

	const sum = PLAYER_OPTIONS.reduce((acc, a) => acc + base[a], 0);
	const choise = Math.floor(Math.random() * sum);
	let runningCount = 0;

	for (const a of PLAYER_OPTIONS) {
		const newSum = runningCount + base[a];
		if (choise > runningCount && choise < newSum) {
			return a;
		}
		runningCount = newSum;
	}

	return PLAYER_OPTIONS[Math.floor(Math.random() * PLAYER_OPTIONS.length)];
}
function CpuSlightlySmartEnd(plays: Play[]) {
	let storageData: Partial<Record<PlayerAction, number>>[] = [{}, {}, {}];
	try {
		// It's ok for this to error
		storageData = JSON.parse(
			localStorage.getItem("slightly-smart-cpu-data") ?? "",
		);
		// Handle the old version of the data
		if (!Array.isArray(storageData)) {
			storageData = [{}, {}, {}];
		}
	} catch {
		// do nothing just goto the default value
	}

	const ResultReverseMappings: Record<PlayerAction, PlayerAction[]> = {
		Scissors: ["Rock", "Spock"],
		Paper: ["Scissors", "Lizard"],
		Rock: ["Paper", "Spock"],
		Lizard: ["Scissors", "Rock"],
		Spock: ["Paper", "Lizard"],
	};

	for (let i = 0; i < 3; i++) {
		ResultReverseMappings[plays[i].user].forEach((a: PlayerAction) => {
			storageData[i][a] = (storageData[i][a] ?? 0) + 1;
		});
	}

	localStorage.setItem(
		"slightly-smart-cpu-data",
		JSON.stringify(storageData),
	);
}
